#include "wios_util.h"



/* GLOBAL FUNCTION */
/* -------------------------------------------------- */

void wios_set_fmtinfo(wios_file_t const * ft, wios_fmtinfo_t * fmt)
{
	if (ft) {
		fmt->channels = ft->fmtinfo.channels;
		fmt->rate     = ft->fmtinfo.rate;
		fmt->format   = ft->fmtinfo.format;
	}
}

void wios_fmtinfo_init(wios_fmtinfo_t * info)
{
	if (info) {
		info->format   = DEFAULT_FORMAT;
		info->channels = DEFAULT_CHANNELS;
		info->rate     = DEFAULT_RATE;
	}
}

void wios_execinfo_init(wios_execinfo_t * info)
{
	if (info) {
		info->periods    = DEFAULT_PERIODS;
		info->periodsize = DEFAULT_PERIODSIZE;
		info->sec        = DEFAULT_EXEC_SEC;
	}
}

/* 再生(録音)時にファイルリード(ライト)とデバイスライト(リード)に使うバッファを確保 */
char * wios_buffer_malloc(wios_file_t * ft, wios_buffer_t * buf, int first_len)
{
	int buf_size = ((first_len*ft->fmtinfo.channels > ft->signal.buf_size) ? first_len*ft->fmtinfo.channels : ft->signal.buf_size);

	if ((buf->fl = (int32_t *)malloc(buf_size*sizeof(int32_t)+DUMMY_BYTE)) == NULL)
		WIOS_ERROR("cannot allocate a buffer for %sing file", ft->io_type == wios_da ? "writ" : "read");

	if ((buf->dv = (char *)malloc(buf_size*sizeof(char)*ft->signal.byte_per_item+DUMMY_BYTE)) == NULL)
		WIOS_ERROR("cannot allocate a buffer for %sing device", ft->io_type == wios_da ? "read" : "writ");
		
	return buf->dv;

error:
	return NULL;
}

void wios_buffer_free(wios_buffer_t * buf)
{
	if (buf) {
		if (buf->fl) {
			free(buf->fl);
			buf->fl = NULL;
		}
		if (buf->dv) {
			free(buf->dv);
			buf->dv = NULL;
		}
	}
}

wios_file_t * open_read_(
	const char             * path,
	const char             * dev,
	const wios_execinfo_t  * info,
	int (*priv_init) (wios_file_t * ft))
{
	wios_file_t * ft;

	if ((ft = malloc(sizeof(wios_file_t))) == NULL)
		WIOS_ERROR("cannot allocate read file structure", NULL);

	ft->io_type = wios_da;
	ft->file_name = path ? path : DEFAULT_DAFILE_NAME;
	ft->dev_name = dev ? dev : ALSA_DEFAULT_DEVICE_NAME;
	if (info) ft->signal.execinfo = *info;
	else wios_execinfo_init(&ft->signal.execinfo);

	if ((*priv_init)(ft) == WIOS_EOF)
		WIOS_ERROR("cannot allocate read file structure", NULL);

	return lsf_open_read(ft);

error:
	return NULL;
}

wios_file_t * open_write_(
		const char             * path,
		const char             * dev,
		const wios_execinfo_t  * info,
		const wios_fmtinfo_t   * fmt,
		int (*priv_init) (wios_file_t * ft))
{
	wios_file_t * ft;

	if ((ft = malloc(sizeof(wios_file_t))) == NULL)
		WIOS_ERROR("cannot allocate write file structure", NULL);

	ft->io_type   = wios_ad;
	ft->file_name = path ? path : DEFAULT_ADFILE_NAME;
	ft->dev_name = dev ? dev : ALSA_DEFAULT_DEVICE_NAME;
	if (fmt) ft->fmtinfo = *fmt;
	else wios_fmtinfo_init(&ft->fmtinfo);
	if (info) ft->signal.execinfo = *info;
	else wios_execinfo_init(&ft->signal.execinfo);

	if ((*priv_init)(ft) == WIOS_EOF)
		WIOS_ERROR("cannot allocate write file structure", NULL);
	
	return lsf_open_write(ft);

error:
	return NULL;
}

char * wios_format_to_string(wios_pcm_format_t fmt)
{
	switch (fmt) {
		case WIOS_PCM_FORMAT_U8  :  return "Unsigned_8bit_PCM";
		case WIOS_PCM_FORMAT_S16 :  return "Signed_16bit_PCM" ;
		case WIOS_PCM_FORMAT_S24 :  return "Signed_24bit_PCM" ;
		case WIOS_PCM_FORMAT_S32 :  return "Signed_32bit_PCM" ;
		default                  :  return "Unknown_format"   ;
	}
}

double wios_get_time(void)
{
  struct timeval tv; 
  gettimeofday(&tv, 0); 
  return tv.tv_sec + tv.tv_usec * 1e-6;
}
